@@ -6,6 +6,9 @@
 import attrdict
 
 from .automata import condition, operation
+from .automata import symbols
+
+SYMBOLS = symbols.SYMBOLS
 
 
 def init_nodes(**kwargs):
@@ -15,30 +18,30 @@ def init_nodes(**kwargs):
 
     # --- initialize condition & operation --- #
     edges = attrdict.AttrDict({
-        'is_end_symbol': condition.IsEndSymbol(),
-        'is_bracket_close': condition.IsBracketClose(),
-        'is_book_close': condition.IsBookClose(),
-        'is_quote_close': condition.IsQuoteClose(),
-        'is_end_state': condition.IsEndState(),
-        'is_long_sentence': condition.IsLongSentence(max_len=max_len),
-        'is_short_sentence': condition.IsShortSentence(min_len=min_len),
+        'is_end_symbol': condition.IsEndSymbolCN(SYMBOLS),
+        'is_bracket_close': condition.IsBracketClose(SYMBOLS),
+        'is_book_close': condition.IsBookClose(SYMBOLS),
+        'is_quote_close': condition.IsQuoteClose(SYMBOLS),
+        'is_end_state': condition.IsEndState(SYMBOLS),
+        'is_long_sentence': condition.IsLongSentence(SYMBOLS, max_len=max_len),
+        'is_short_sentence': condition.IsShortSentence(SYMBOLS, min_len=min_len),
         # --- sepcial Condition --- #
-        'is_right_quota': condition.IsRightQuota(),
-        'is_left_quota': condition.IsLeftQuota(index=1),
+        'is_right_quota': condition.IsRightQuotaZH(),
+        'is_left_quota': condition.IsLeftQuotaZH(index=1),
         'is_leftQ_gt_1': condition.IsLeftQuotaGreaterThan(theta=1),
         'is_rightQ_before_leftQ': condition.IsRQuotaStickWithLQuota(),
-        'is_empty': condition.IsSpace(),
+        'is_empty': condition.IsSpace(SYMBOLS),
         'with_says': condition.WithSays(),
         'special_ends': condition.SpecialEnds()
     })
 
     nodes = attrdict.AttrDict({
-        'do_cut': operation.Normal(),
-        'long_handler': operation.LongHandler(hard_max=hard_max),
-        'short_handler': operation.ShortHandler(),
-        'init': operation.Indolent(),
-        'quota_handler': operation.Indolent(),
-        'end': operation.EndState()
+        'do_cut': operation.Normal(SYMBOLS),
+        'long_handler': operation.LongHandler(SYMBOLS, hard_max=hard_max),
+        'short_handler': operation.ShortHandler(SYMBOLS),
+        'init': operation.Indolent(SYMBOLS),
+        'quota_handler': operation.Indolent(SYMBOLS),
+        'end': operation.EndState(SYMBOLS)
     })
     return edges, nodes
 
